@@ -1,3 +1,7 @@
+// Has false positives on else if chains that sometimes have the same
+// body for readability.
+#![allow(clippy::if_same_then_else)]
+
 use rayon::prelude::*;
 use std::path::PathBuf;
 use version_check as rustc;
@@ -107,7 +111,7 @@ impl TreeSitterParser {
         }
 
         let mut build = cc::Build::new();
-        if cfg!(windows) {
+        if cfg!(target_env = "msvc") {
             build.flag("/utf-8");
         }
         build.include(&dir).warnings(false); // ignore unused parameter warnings
