@@ -2,14 +2,18 @@
 
 use owo_colors::{OwoColorize, Style};
 use rustc_hash::FxHashMap;
-use std::{cmp::max, collections::HashSet};
+use std::{
+    cmp::{max, min},
+    collections::HashSet,
+};
 
 use crate::{
     constants::Side,
     display::context::all_matched_lines_filled,
     display::hunks::{matched_lines_indexes_for_hunk, Hunk},
     display::style::{
-        self, apply_colors, color_positions, novel_style, split_and_apply, BackgroundColor,
+        self, apply_colors, apply_line_number_color, color_positions, novel_style, split_and_apply,
+        BackgroundColor,
     },
     lines::{codepoint_len, format_line_num, split_on_newlines, LineNumber},
     options::{DisplayMode, DisplayOptions},
@@ -33,19 +37,17 @@ fn format_line_num_padded(line_num: LineNumber, column_width: usize) -> String {
 fn format_missing_line_num(
     prev_num: LineNumber,
     source_dims: &SourceDimensions,
-    is_lhs: bool,
+    side: Side,
     use_color: bool,
 ) -> String {
-    let column_width = if is_lhs {
-        source_dims.lhs_line_nums_width
-    } else {
-        source_dims.rhs_line_nums_width
+    let column_width = match side {
+        Side::Left => source_dims.lhs_line_nums_width,
+        Side::Right => source_dims.rhs_line_nums_width,
     };
 
-    let after_end = if is_lhs {
-        prev_num >= source_dims.lhs_max_line
-    } else {
-        prev_num >= source_dims.rhs_max_line
+    let after_end = match side {
+        Side::Left => prev_num >= source_dims.lhs_max_line,
+        Side::Right => prev_num >= source_dims.rhs_max_line,
     };
 
     let mut style = Style::new();
@@ -69,7 +71,7 @@ fn display_single_column(
     rhs_display_path: &str,
     lang_name: &str,
     src_lines: &[String],
-    is_lhs: bool,
+    side: Side,
     display_options: &DisplayOptions,
 ) -> Vec<String> {
     let column_width = format_line_num((src_lines.len() as u32).into()).len();
@@ -90,7 +92,7 @@ fn display_single_column(
 
     let mut style = Style::new();
     if display_options.use_color {
-        style = novel_style(Style::new(), is_lhs, display_options.background_color);
+        style = novel_style(Style::new(), side, display_options.background_color);
     }
 
     for (i, line) in src_lines.iter().enumerate() {
@@ -111,8 +113,7 @@ fn display_line_nums(
     lhs_line_num: Option<LineNumber>,
     rhs_line_num: Option<LineNumber>,
     source_dims: &SourceDimensions,
-    use_color: bool,
-    background: BackgroundColor,
+    display_options: &DisplayOptions,
     lhs_has_novel: bool,
     rhs_has_novel: bool,
     prev_lhs_line_num: Option<LineNumber>,
@@ -121,42 +122,25 @@ fn display_line_nums(
     let display_lhs_line_num: String = match lhs_line_num {
         Some(line_num) => {
             let s = format_line_num_padded(line_num, source_dims.lhs_line_nums_width);
-            if lhs_has_novel && use_color {
-                // TODO: factor out applying colours to line numbers.
-                if background.is_dark() {
-                    s.bright_red().to_string()
-                } else {
-                    s.red().to_string()
-                }
-            } else {
-                s
-            }
+            apply_line_number_color(&s, lhs_has_novel, Side::Left, display_options)
         }
         None => format_missing_line_num(
             prev_lhs_line_num.unwrap_or_else(|| 1.into()),
             source_dims,
-            true,
-            use_color,
+            Side::Left,
+            display_options.use_color,
         ),
     };
     let display_rhs_line_num: String = match rhs_line_num {
         Some(line_num) => {
             let s = format_line_num_padded(line_num, source_dims.rhs_line_nums_width);
-            if rhs_has_novel && use_color {
-                if background.is_dark() {
-                    s.bright_green().to_string()
-                } else {
-                    s.green().to_string()
-                }
-            } else {
-                s
-            }
+            apply_line_number_color(&s, rhs_has_novel, Side::Right, display_options)
         }
         None => format_missing_line_num(
             prev_rhs_line_num.unwrap_or_else(|| 1.into()),
             source_dims,
-            false,
-            use_color,
+            Side::Right,
+            display_options.use_color,
         ),
     };
 
@@ -166,11 +150,18 @@ fn display_line_nums(
 // Sizes used when displaying a hunk.
 #[derive(Debug)]
 struct SourceDimensions {
-    lhs_content_width: usize,
-    rhs_content_width: usize,
+    /// The number of characters used to display source lines. Any
+    /// line that exceeds this length will be wrapped.
+    content_width: usize,
+    /// The number of characters required to display line numbers on
+    /// the LHS.
     lhs_line_nums_width: usize,
+    /// The number of characters required to display line numbers on
+    /// the RHS.
     rhs_line_nums_width: usize,
+    /// The highest line number in the LHS source.
     lhs_max_line: LineNumber,
+    /// The highest line number in the RHS source.
     rhs_max_line: LineNumber,
 }
 
@@ -225,9 +216,13 @@ impl SourceDimensions {
                 - rhs_line_nums_width as isize,
         ) as usize;
 
+        // We want the content width to be the same on both
+        // sides. This ensures that line wrapping splits lines at the
+        // same point on both sides.
+        let content_width = min(lhs_content_width, rhs_content_width);
+
         Self {
-            lhs_content_width,
-            rhs_content_width,
+            content_width,
             lhs_line_nums_width,
             rhs_line_nums_width,
             lhs_max_line,
@@ -266,7 +261,8 @@ fn highlight_positions(
     FxHashMap<LineNumber, Vec<(SingleLineSpan, Style)>>,
     FxHashMap<LineNumber, Vec<(SingleLineSpan, Style)>>,
 ) {
-    let lhs_positions = color_positions(true, background, syntax_highlight, language, lhs_mps);
+    let lhs_positions =
+        color_positions(Side::Left, background, syntax_highlight, language, lhs_mps);
     // Preallocate the hashmap assuming the average line will have 2 items on it.
     let mut lhs_styles: FxHashMap<LineNumber, Vec<(SingleLineSpan, Style)>> = FxHashMap::default();
     for (span, style) in lhs_positions {
@@ -274,7 +270,8 @@ fn highlight_positions(
         styles.push((span, style));
     }
 
-    let rhs_positions = color_positions(false, background, syntax_highlight, language, rhs_mps);
+    let rhs_positions =
+        color_positions(Side::Right, background, syntax_highlight, language, rhs_mps);
     let mut rhs_styles: FxHashMap<LineNumber, Vec<(SingleLineSpan, Style)>> = FxHashMap::default();
     for (span, style) in rhs_positions {
         let styles = rhs_styles.entry(span.line).or_insert_with(Vec::new);
@@ -324,7 +321,7 @@ pub fn print(
         (
             apply_colors(
                 lhs_src,
-                true,
+                Side::Left,
                 display_options.syntax_highlight,
                 language,
                 display_options.background_color,
@@ -332,7 +329,7 @@ pub fn print(
             ),
             apply_colors(
                 rhs_src,
-                false,
+                Side::Right,
                 display_options.syntax_highlight,
                 language,
                 display_options.background_color,
@@ -358,7 +355,7 @@ pub fn print(
             rhs_display_path,
             lang_name,
             &rhs_colored_lines,
-            false,
+            Side::Right,
             display_options,
         ) {
             print!("{}", line);
@@ -372,7 +369,7 @@ pub fn print(
             rhs_display_path,
             lang_name,
             &lhs_colored_lines,
-            true,
+            Side::Left,
             display_options,
         ) {
             print!("{}", line);
@@ -458,8 +455,7 @@ pub fn print(
                 *lhs_line_num,
                 *rhs_line_num,
                 &source_dims,
-                display_options.use_color,
-                display_options.background_color,
+                display_options,
                 lhs_line_novel,
                 rhs_line_novel,
                 prev_lhs_line_num,
@@ -511,17 +507,17 @@ pub fn print(
                 let lhs_line = match lhs_line_num {
                     Some(lhs_line_num) => split_and_apply(
                         lhs_lines[lhs_line_num.as_usize()],
-                        source_dims.lhs_content_width,
+                        source_dims.content_width,
                         display_options.use_color,
                         lhs_highlights.get(lhs_line_num).unwrap_or(&vec![]),
                         Side::Left,
                     ),
-                    None => vec![" ".repeat(source_dims.lhs_content_width)],
+                    None => vec![" ".repeat(source_dims.content_width)],
                 };
                 let rhs_line = match rhs_line_num {
                     Some(rhs_line_num) => split_and_apply(
                         rhs_lines[rhs_line_num.as_usize()],
-                        source_dims.rhs_content_width,
+                        source_dims.content_width,
                         display_options.use_color,
                         rhs_highlights.get(rhs_line_num).unwrap_or(&vec![]),
                         Side::Right,
@@ -534,7 +530,7 @@ pub fn print(
                     .enumerate()
                 {
                     let lhs_line =
-                        lhs_line.unwrap_or_else(|| " ".repeat(source_dims.lhs_content_width));
+                        lhs_line.unwrap_or_else(|| " ".repeat(source_dims.content_width));
                     let rhs_line = rhs_line.unwrap_or_else(|| "".into());
                     let lhs_num: String = if i == 0 {
                         display_lhs_line_num.clone()
@@ -543,18 +539,16 @@ pub fn print(
                             lhs_line_num
                                 .unwrap_or_else(|| prev_lhs_line_num.unwrap_or_else(|| 10.into())),
                             &source_dims,
-                            true,
+                            Side::Left,
                             display_options.use_color,
                         );
                         if let Some(line_num) = lhs_line_num {
-                            if display_options.use_color && lhs_lines_with_novel.contains(line_num)
-                            {
-                                s = if display_options.background_color.is_dark() {
-                                    s.bright_red().to_string()
-                                } else {
-                                    s.red().to_string()
-                                };
-                            }
+                            s = apply_line_number_color(
+                                &s,
+                                lhs_lines_with_novel.contains(line_num),
+                                Side::Left,
+                                display_options,
+                            );
                         }
                         s
                     };
@@ -565,18 +559,16 @@ pub fn print(
                             rhs_line_num
                                 .unwrap_or_else(|| prev_rhs_line_num.unwrap_or_else(|| 10.into())),
                             &source_dims,
-                            false,
+                            Side::Right,
                             display_options.use_color,
                         );
                         if let Some(line_num) = rhs_line_num {
-                            if display_options.use_color && rhs_lines_with_novel.contains(line_num)
-                            {
-                                s = if display_options.background_color.is_dark() {
-                                    s.bright_green().to_string()
-                                } else {
-                                    s.green().to_string()
-                                };
-                            }
+                            s = apply_line_number_color(
+                                &s,
+                                rhs_lines_with_novel.contains(line_num),
+                                Side::Right,
+                                display_options,
+                            );
                         }
                         s
                     };
@@ -630,11 +622,11 @@ mod tests {
         );
 
         assert_eq!(
-            format_missing_line_num(0.into(), &source_dims, true, true),
+            format_missing_line_num(0.into(), &source_dims, Side::Left, true),
             ". ".dimmed().to_string()
         );
         assert_eq!(
-            format_missing_line_num(0.into(), &source_dims, true, false),
+            format_missing_line_num(0.into(), &source_dims, Side::Left, false),
             ". ".to_string()
         );
     }
@@ -652,11 +644,11 @@ mod tests {
         );
 
         assert_eq!(
-            format_missing_line_num(1.into(), &source_dims, true, true),
+            format_missing_line_num(1.into(), &source_dims, Side::Left, true),
             "  ".dimmed().to_string()
         );
         assert_eq!(
-            format_missing_line_num(1.into(), &source_dims, true, false),
+            format_missing_line_num(1.into(), &source_dims, Side::Left, false),
             "  ".to_string()
         );
     }
@@ -681,7 +673,7 @@ mod tests {
             "foo.py",
             "Python",
             &["print(123)\n".to_string()],
-            false,
+            Side::Right,
             &display_options,
         );
         let res = res_lines.join("");
