@@ -257,22 +257,22 @@ fn style_lines(lines: &[&str], styles: &[(SingleLineSpan, Style)]) -> Vec<String
     res
 }
 
-pub fn novel_style(style: Style, is_lhs: bool, background: BackgroundColor) -> Style {
+pub fn novel_style(style: Style, side: Side, background: BackgroundColor) -> Style {
     if background.is_dark() {
-        if is_lhs {
-            style.bright_red()
-        } else {
-            style.bright_green()
+        match side {
+            Side::Left => style.bright_red(),
+            Side::Right => style.bright_green(),
         }
-    } else if is_lhs {
-        style.red()
     } else {
-        style.green()
+        match side {
+            Side::Left => style.red(),
+            Side::Right => style.green(),
+        }
     }
 }
 
 pub fn color_positions(
-    is_lhs: bool,
+    side: Side,
     background: BackgroundColor,
     syntax_highlight: bool,
     language: Option<Language>,
@@ -311,7 +311,7 @@ pub fn color_positions(
                 }
             }
             MatchKind::Novel { highlight, .. } => {
-                style = novel_style(style, is_lhs, background);
+                style = novel_style(style, side, background);
                 if syntax_highlight
                     && matches!(
                         highlight,
@@ -327,7 +327,7 @@ pub fn color_positions(
                 }
             }
             MatchKind::NovelWord { highlight } => {
-                style = novel_style(style, is_lhs, background).bold();
+                style = novel_style(style, side, background).bold();
 
                 // Underline novel words inside comments in code, but
                 // don't apply it to every single line in plaintext.
@@ -340,7 +340,7 @@ pub fn color_positions(
                 }
             }
             MatchKind::NovelLinePart { highlight, .. } => {
-                style = novel_style(style, is_lhs, background);
+                style = novel_style(style, side, background);
                 if syntax_highlight && matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
                     style = style.italic();
                 }
@@ -353,13 +353,13 @@ pub fn color_positions(
 
 pub fn apply_colors(
     s: &str,
-    is_lhs: bool,
+    side: Side,
     syntax_highlight: bool,
     language: Option<Language>,
     background: BackgroundColor,
     positions: &[MatchedPos],
 ) -> Vec<String> {
-    let styles = color_positions(is_lhs, background, syntax_highlight, language, positions);
+    let styles = color_positions(side, background, syntax_highlight, language, positions);
     let lines = split_on_newlines(s);
     style_lines(&lines, &styles)
 }
@@ -378,6 +378,34 @@ fn apply_header_color(s: &str, use_color: bool, background: BackgroundColor) ->
     }
 }
 
+pub(crate) fn apply_line_number_color(
+    s: &str,
+    is_novel: bool,
+    side: Side,
+    display_options: &DisplayOptions,
+) -> String {
+    if display_options.use_color {
+        let mut style = Style::new();
+
+        // The goal here is to choose a style for line numbers that is
+        // visually distinct from content.
+        if is_novel {
+            // For changed lines, show the line number as red/green
+            // and bold. This works well for syntactic diffs, where
+            // most content is not bold.
+            style = novel_style(style, side, display_options.background_color).bold();
+        } else {
+            // For unchanged lines, dim the line numbers so it's
+            // clearly separate from the content.
+            style = style.dimmed()
+        }
+
+        s.style(style).to_string()
+    } else {
+        s.to_string()
+    }
+}
+
 pub fn header(
     lhs_display_path: &str,
     rhs_display_path: &str,
