@@ -15,6 +15,9 @@
 // Has false positives on else if chains that sometimes have the same
 // body for readability.
 #![allow(clippy::if_same_then_else)]
+// Purely stylistic, and ignores whether there are explanatory
+// comments in the if/else.
+#![allow(clippy::bool_to_int_with_if)]
 
 mod constants;
 mod diff;
