@@ -1,6 +1,6 @@
 //! CLI option parsing.
 
-use std::{borrow::Borrow, env, ffi::OsString};
+use std::{borrow::Borrow, env, ffi::OsStr, path::PathBuf};
 
 use atty::Stream;
 use clap::{crate_authors, crate_description, crate_version, Arg, Command};
@@ -32,6 +32,7 @@ pub struct DisplayOptions {
     pub print_unchanged: bool,
     pub tab_width: usize,
     pub display_width: usize,
+    pub num_context_lines: u32,
     pub in_vcs: bool,
     pub syntax_highlight: bool,
 }
@@ -72,6 +73,17 @@ fn app() -> clap::Command<'static> {
                     "Parse a single file with tree-sitter and display the tree-sitter parse tree.",
                 ).help_heading("DEBUG OPTIONS"),
         )
+        .arg(
+            Arg::new("context")
+                .long("context")
+                .takes_value(true)
+                .value_name("LINES")
+                .long_help("The number of contextual lines to show around changed lines.")
+                .default_value("3")
+                .env("DFT_CONTEXT")
+                .validator(|s| s.parse::<u32>())
+                .required(false),
+        )
         .arg(
             Arg::new("width")
                 .long("width")
@@ -178,6 +190,41 @@ pub enum DisplayMode {
     SideBySideShowBoth,
 }
 
+#[derive(Eq, PartialEq, Debug)]
+pub enum FileArgument {
+    NamedPath(std::path::PathBuf),
+    Stdin,
+    DevNull,
+}
+
+impl FileArgument {
+    /// Return a `FileArgument` representing this command line
+    /// argument.
+    pub fn from_cli_argument(arg: &OsStr) -> Self {
+        if arg == "/dev/null" {
+            FileArgument::DevNull
+        } else if arg == "-" {
+            FileArgument::Stdin
+        } else {
+            FileArgument::NamedPath(PathBuf::from(arg))
+        }
+    }
+
+    /// Return a `FileArgument` that always represents a path that
+    /// exists.
+    pub fn from_path_argument(arg: &OsStr) -> Self {
+        FileArgument::NamedPath(PathBuf::from(arg))
+    }
+
+    pub fn display(&self) -> String {
+        match self {
+            FileArgument::NamedPath(path) => path.display().to_string(),
+            FileArgument::Stdin => "(stdin)".to_string(),
+            FileArgument::DevNull => "/dev/null".to_string(),
+        }
+    }
+}
+
 pub enum Mode {
     Diff {
         graph_limit: usize,
@@ -187,16 +234,20 @@ pub enum Mode {
         language_override: Option<guess_language::Language>,
         /// The path where we can read the LHS file. This is often a
         /// temporary file generated by source control.
-        lhs_path: OsString,
+        lhs_path: FileArgument,
         /// The path where we can read the RHS file. This is often a
         /// temporary file generated by source control.
-        rhs_path: OsString,
-        /// The path that we should display for the LHS file.
+        rhs_path: FileArgument,
+        /// The path that we should display for the LHS file. This is
+        /// usually the same as `lhs_path`, but the six argument form
+        /// of git-diff specifies this separately.
         lhs_display_path: String,
         /// The path that we should display for the RHS file.
         rhs_display_path: String,
     },
-    ListLanguages,
+    ListLanguages {
+        use_color: bool,
+    },
     DumpTreeSitter {
         path: String,
         language_override: Option<guess_language::Language>,
@@ -226,8 +277,22 @@ pub fn parse_args() -> Mode {
         None => None,
     };
 
+    let color_output = if let Some(color_when) = matches.value_of("color") {
+        if color_when == "always" {
+            ColorOutput::Always
+        } else if color_when == "never" {
+            ColorOutput::Never
+        } else {
+            ColorOutput::Auto
+        }
+    } else {
+        ColorOutput::Auto
+    };
+
+    let use_color = should_use_color(color_output);
+
     if matches.is_present("list-languages") {
-        return Mode::ListLanguages;
+        return Mode::ListLanguages { use_color };
     }
 
     if let Some(path) = matches.value_of("dump-syntax") {
@@ -249,20 +314,24 @@ pub fn parse_args() -> Mode {
 
     // TODO: document these different ways of calling difftastic.
     let (lhs_display_path, rhs_display_path, lhs_path, rhs_path, in_vcs) = match &args[..] {
-        [lhs_path, rhs_path] => (
-            lhs_path.to_owned(),
-            rhs_path.to_owned(),
-            lhs_path.to_owned(),
-            rhs_path.to_owned(),
-            false,
-        ),
+        [lhs_path, rhs_path] => {
+            let lhs_arg = FileArgument::from_cli_argument(lhs_path);
+            let rhs_arg = FileArgument::from_cli_argument(rhs_path);
+            (
+                lhs_arg.display(),
+                rhs_arg.display(),
+                lhs_arg,
+                rhs_arg,
+                false,
+            )
+        }
         [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
             // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
             (
-                display_path.to_owned(),
-                display_path.to_owned(),
-                lhs_tmp_file.to_owned(),
-                rhs_tmp_file.to_owned(),
+                display_path.to_string_lossy().to_string(),
+                display_path.to_string_lossy().to_string(),
+                FileArgument::from_path_argument(lhs_tmp_file),
+                FileArgument::from_path_argument(rhs_tmp_file),
                 true,
             )
         }
@@ -271,10 +340,10 @@ pub fn parse_args() -> Mode {
             // Rename file.
             // TODO: where does git document these 9 arguments?
             (
-                old_name.to_owned(),
-                new_name.to_owned(),
-                lhs_tmp_file.to_owned(),
-                rhs_tmp_file.to_owned(),
+                old_name.to_string_lossy().to_string(),
+                new_name.to_string_lossy().to_string(),
+                FileArgument::from_path_argument(lhs_tmp_file),
+                FileArgument::from_path_argument(rhs_tmp_file),
                 true,
             )
         }
@@ -313,18 +382,6 @@ pub fn parse_args() -> Mode {
         DisplayMode::SideBySide
     };
 
-    let color_output = if let Some(color_when) = matches.value_of("color") {
-        if color_when == "always" {
-            ColorOutput::Always
-        } else if color_when == "never" {
-            ColorOutput::Never
-        } else {
-            ColorOutput::Auto
-        }
-    } else {
-        ColorOutput::Auto
-    };
-
     let background_color = match matches
         .value_of("background")
         .expect("Always present as we've given clap a default")
@@ -354,10 +411,17 @@ pub fn parse_args() -> Mode {
         .parse::<usize>()
         .expect("Value already validated by clap");
 
-    let print_unchanged = !matches.is_present("skip-unchanged");
-    let missing_as_empty = matches.is_present("missing-as-empty");
+    let num_context_lines = matches
+        .value_of("context")
+        .expect("Always present as we've given clap a default")
+        .parse::<u32>()
+        .expect("Value already validated by clap");
 
-    let use_color = should_use_color(color_output);
+    let print_unchanged = !matches.is_present("skip-unchanged");
+
+    // TODO: is this necessary now we handle /dev/null as an empty
+    // file on all platforms?
+    let missing_as_empty = matches.is_present("missing-as-empty");
 
     let display_options = DisplayOptions {
         background_color,
@@ -366,6 +430,7 @@ pub fn parse_args() -> Mode {
         tab_width,
         display_mode,
         display_width,
+        num_context_lines,
         syntax_highlight,
         in_vcs,
     };
@@ -376,10 +441,10 @@ pub fn parse_args() -> Mode {
         display_options,
         missing_as_empty,
         language_override,
-        lhs_path: lhs_path.to_owned(),
-        rhs_path: rhs_path.to_owned(),
-        lhs_display_path: lhs_display_path.to_string_lossy().to_string(),
-        rhs_display_path: rhs_display_path.to_string_lossy().to_string(),
+        lhs_path,
+        rhs_path,
+        lhs_display_path,
+        rhs_display_path,
     }
 }
 
