@@ -1077,15 +1077,15 @@ fn syntax_from_cursor<'a>(
     if node.is_error() {
         let position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         let content = &src[node.start_byte()..node.end_byte()];
-
         debug!(
             "Tree-sitter syntax error at {:?}: {}",
             position.get(0),
             content
         );
-    }
 
-    if config.atom_nodes.contains(node.kind()) {
+        // Treat error nodes as atoms, even if they have children.
+        atom_from_cursor(arena, src, nl_pos, cursor, highlights)
+    } else if config.atom_nodes.contains(node.kind()) {
         // Treat nodes like string literals as atoms, regardless
         // of whether they have children.
         atom_from_cursor(arena, src, nl_pos, cursor, highlights)
